@@ -10,8 +10,10 @@ from django.utils.translation import gettext_lazy as _
 
 from ipam.models import IPAddress
 from netbox.admin import admin_site
-from .background_tasks import update_dns
+from netbox_ddns.models import DNSStatus, ExtraDNSName
+from .background_tasks import dns_create
 from .models import ReverseZone, Server, Zone
+from .utils import normalize_fqdn
 
 logger = logging.getLogger('netbox_ddns')
 
@@ -65,20 +67,46 @@ class ZoneAdmin(admin.ModelAdmin):
             more_specifics = Zone.objects.filter(name__endswith=zone.name).exclude(pk=zone.pk)
 
             # Find all IPAddress objects in this zone but not in the more-specifics
-            addresses = IPAddress.objects.filter(Q(dns_name__endswith=zone.name) |
-                                                 Q(dns_name__endswith=zone.name.rstrip('.')))
+            ip_addresses = IPAddress.objects.filter(Q(dns_name__endswith=zone.name) |
+                                                    Q(dns_name__endswith=zone.name.rstrip('.')))
             for more_specific in more_specifics:
-                addresses = addresses.exclude(Q(dns_name__endswith=more_specific.name) |
-                                              Q(dns_name__endswith=more_specific.name.rstrip('.')))
+                ip_addresses = ip_addresses.exclude(Q(dns_name__endswith=more_specific.name) |
+                                                    Q(dns_name__endswith=more_specific.name.rstrip('.')))
 
-            for address in addresses:
-                if address.dns_name:
-                    update_dns.delay(
-                        new_record=address,
-                        skip_reverse=True
+            for ip_address in ip_addresses:
+                new_address = ip_address.address.ip
+                new_dns_name = normalize_fqdn(ip_address.dns_name)
+
+                if new_dns_name:
+                    status, created = DNSStatus.objects.get_or_create(ip_address=ip_address)
+
+                    dns_create.delay(
+                        dns_name=new_dns_name,
+                        address=new_address,
+                        status=status,
+                        reverse=False,
                     )
+
                     counter += 1
 
+            # Find all ExtraDNSName objects in this zone but not in the more-specifics
+            extra_names = ExtraDNSName.objects.filter(name__endswith=zone.name)
+            for more_specific in more_specifics:
+                extra_names = extra_names.exclude(name__endswith=more_specific.name)
+
+            for extra in extra_names:
+                new_address = extra.ip_address.address.ip
+                new_dns_name = extra.name
+
+                dns_create.delay(
+                    dns_name=new_dns_name,
+                    address=new_address,
+                    status=extra,
+                    reverse=False,
+                )
+
+                counter += 1
+
             messages.info(request, _("Updating {count} forward records in {name}").format(count=counter,
                                                                                           name=zone.name))
 
@@ -99,17 +127,30 @@ class ReverseZoneAdmin(admin.ModelAdmin):
             more_specifics = ReverseZone.objects.filter(prefix__net_contained=zone.prefix).exclude(pk=zone.pk)
 
             # Find all IPAddress objects in this zone but not in the more-specifics
-            addresses = IPAddress.objects.filter(address__net_contained=zone.prefix)
+            ip_addresses = IPAddress.objects.filter(address__net_contained=zone.prefix)
             for more_specific in more_specifics:
-                addresses = addresses.exclude(address__net_contained=more_specific.prefix)
+                ip_addresses = ip_addresses.exclude(address__net_contained=more_specific.prefix)
 
-            for address in addresses:
-                if address.dns_name:
-                    update_dns.delay(
-                        new_record=address,
-                        skip_forward=True
+            for ip_address in ip_addresses:
+                new_address = ip_address.address.ip
+                new_dns_name = normalize_fqdn(ip_address.dns_name)
+
+                if new_dns_name:
+                    status, created = DNSStatus.objects.get_or_create(ip_address=ip_address)
+
+                    dns_create.delay(
+                        dns_name=new_dns_name,
+                        address=new_address,
+                        status=status,
+                        forward=False,
                     )
+
                     counter += 1
 
             messages.info(request, _("Updating {count} reverse records in {name}").format(count=counter,
                                                                                           name=zone.name))
+
+
+@admin.register(ExtraDNSName, site=admin_site)
+class ExtraDNSNameAdmin(admin.ModelAdmin):
+    pass
